use ducklang::interpreter::machine::Machine;
use ducklang::parser::ast::build_ast;
use std::{env, fs::File, io::Read, process::exit};

// better cli
// --debug flag
pub fn main() {
    let path = env::args().nth(1).expect("no file provided");
    let mut f = File::open(&path).unwrap_or_else(|_| panic!("file {} not found", path));
    let mut content = String::new();
    f.read_to_string(&mut content)
        .unwrap_or_else(|_| panic!("Error in reading file {}", path));
    let ast = match build_ast(&content) {
        Ok(ast) => ast,
        Err(e) => {
            eprintln!("Error in parsing file {}: {}", path, e);
            exit(1);
        }
    };
    let mut machine = Machine::new();
    match machine.run(ast) {
        Ok(ret) => exit(ret as i32),
        Err(e) => {
            eprintln!("Runtime error: {}", e);
            exit(1)
        }
    };
}
