use super::ast::Rule;

use pest::iterators::Pair;

// get parsing errors out of this file
// create a error print for DuckParser error
// todo => put filename
pub fn parsing_error(pair: Pair<Rule>, error: &str) -> String {
    let span = pair.as_span();

    let (err_line, err_col) = span.start_pos().line_col();
    let col_whitespaces = " ".repeat(err_line.to_string().len());

    let span_len = span.end() - span.start();
    let underline_chars = "^".repeat(span_len);

    let error_print = vec![
        format!("\nParsing error at --> {}:{}", err_line, err_col),
        format!("{} |", col_whitespaces),
        format!("{} | {}", err_line, span.as_str()),
        format!("{} | {}", col_whitespaces, underline_chars),
        format!("{} |", col_whitespaces),
        format!("{} = {}", col_whitespaces, error),
    ];
    error_print.join("\n")
}

pub fn renamed_parsing_error_rules(rule: &Rule) -> String {
    match *rule {
        Rule::program => "program".to_owned(),
        Rule::stat => "statement".to_owned(),
        Rule::stat_assign => "assignment".to_owned(),
        Rule::stat_if => "if statement".to_owned(),
        Rule::if_keyword => "if keyword".to_owned(),
        Rule::else_keyword => "else keyword".to_owned(),
        Rule::stat_elif => "elif statement".to_owned(),
        Rule::else_if_keyword => "else if keyword".to_owned(),
        Rule::stat_while => "while statement".to_owned(),
        Rule::while_keyword => "while keyword".to_owned(),
        Rule::stat_break => "break statement".to_owned(),
        Rule::break_keyword => "break keyword".to_owned(),
        Rule::stat_continue => "continue statement".to_owned(),
        Rule::continue_keyword => "continue keyword".to_owned(),
        Rule::stat_define => "define statement".to_owned(),
        Rule::stat_return => "return statement".to_owned(),
        Rule::return_keyword => "return keyword".to_owned(),
        Rule::stat_print => "print statement".to_owned(),
        Rule::print_keyword => "print keyword".to_owned(),
        Rule::end_of_stat => ";".to_owned(),
        Rule::expr => "expression".to_owned(),
        Rule::term => "term".to_owned(),
        Rule::call => "function call".to_owned(),
        Rule::variable => "variable".to_owned(),
        Rule::number => "number".to_owned(),
        Rule::boolean => "boolean".to_owned(),
        Rule::op_assign => "=".to_owned(),
        Rule::op_or => "||".to_owned(),
        Rule::op_and => "&&".to_owned(),
        Rule::op_lt => "<".to_owned(),
        Rule::op_gt => ">".to_owned(),
        Rule::op_eq => "==".to_owned(),
        Rule::op_ge => ">=".to_owned(),
        Rule::op_le => "<=".to_owned(),
        Rule::op_ne => "!=".to_owned(),
        Rule::op_add => "+".to_owned(),
        Rule::op_sub => "-".to_owned(),
        Rule::op_mul => "*".to_owned(),
        Rule::op_div => "/".to_owned(),
        Rule::op_mod => "%".to_owned(),
        Rule::op_neg => "-".to_owned(),
        Rule::op_not => "!".to_owned(),
        Rule::op_index => "[]".to_owned(),
        Rule::string => "string".to_owned(),
        Rule::ascii_string => "ascii string".to_owned(),
        Rule::ascii_range => "ascii range".to_owned(),
        Rule::WHITESPACE => "whitespace".to_owned(),
        Rule::COMMENT => "comment".to_owned(),
        Rule::EOI => "end of input".to_owned(),
        r => format!("{:?}", r),
    }
}
