#[derive(Debug, PartialEq)]
pub enum EscapeError {
    EscapeAtEndOfString,
    InvalidEscapedChar(char),
}

struct InterpretEscapedString<'a> {
    s: std::str::Chars<'a>,
}

fn convert_escape(c: char) -> Result<u8, EscapeError> {
    match c {
        '0' => Ok(b'\0'),
        'a' => Ok(b'\x07'),
        'b' => Ok(b'\x08'),
        'f' => Ok(b'\x0c'),
        'n' => Ok(b'\n'),
        'r' => Ok(b'\r'),
        't' => Ok(b'\t'),
        'v' => Ok(b'\x0b'),
        '\'' => Ok(b'\''),
        '"' => Ok(b'"'),
        '?' => Ok(b'?'),
        '\\' => Ok(b'\\'),
        c => Err(EscapeError::InvalidEscapedChar(c)),
    }
}

impl<'a> Iterator for InterpretEscapedString<'a> {
    type Item = Result<u8, EscapeError>;

    fn next(&mut self) -> Option<Self::Item> {
        self.s.next().map(|c| match c {
            '\\' => match self.s.next() {
                Some(c) => convert_escape(c),
                None => Err(EscapeError::EscapeAtEndOfString),
            },
            c => Ok(c as u8),
        })
    }
}

pub fn interpret_escaped_string(s: &str) -> Result<Vec<u8>, EscapeError> {
    (InterpretEscapedString { s: s.chars() }).collect()
}

pub fn interpret_escaped_char(s: &str) -> Result<u8, EscapeError> {
    match (InterpretEscapedString { s: s.chars() }).next() {
        Some(res) => res,
        None => Err(EscapeError::EscapeAtEndOfString),
    }
}
