pub mod ast;
pub mod error;
pub mod escape;
pub mod syntax;

#[cfg(test)]
mod tests {
    use crate::parser::ast::build_ast;
    use std::{fs::File, io::Read};

    fn parse_file(path: &str) {
        let mut f = File::open(path).expect(&format!("file {} not found", path));
        let mut content = String::new();
        f.read_to_string(&mut content)
            .expect(&format!("Error in reading file {}", path));
        build_ast(&content).unwrap();
    }

    // TODO test_directly the ast of a program
    #[test]
    fn test_define_assign() {
        parse_file("test_programs/define_assign.duck");
    }

    #[test]
    fn test_unknown_variable() {
        parse_file("test_programs/unknown_variable.duck");
    }

    #[test]
    fn double_define_fn() {
        parse_file("test_programs/double_define_fn.duck");
    }

    #[test]
    fn test_double_define_var() {
        parse_file("test_programs/double_define_var.duck");
    }

    #[test]
    fn test_arithmetic_expr() {
        parse_file("test_programs/arithmetic_expr.duck");
    }

    #[test]
    fn test_boolean_expr() {
        parse_file("test_programs/boolean_expr.duck");
    }

    #[test]
    fn test_compare_expr() {
        parse_file("test_programs/compare_expr.duck");
    }

    #[test]
    fn test_conditional() {
        parse_file("test_programs/conditional.duck");
    }

    #[test]
    fn test_while_loop() {
        parse_file("test_programs/while_loop.duck");
    }

    #[test]
    #[should_panic]
    fn test_break_not_loop() {
        parse_file("test_programs/break_not_loop.duck");
    }

    #[test]
    fn test_functions() {
        parse_file("test_programs/functions.duck");
    }

    #[test]
    fn test_string_char() {
        parse_file("test_programs/string_char.duck");
    }

    #[test]
    fn test_local_block_env() {
        parse_file("test_programs/local_block_env.duck");
    }

    #[test]
    fn test_arrays() {
        parse_file("test_programs/arrays.duck");
    }
}
